function rules = associationRules(freqPatt,minConfidence, tranDb)
% rules = associationRules(freqPatt,minConfidence)
%-----------------------------------------------------------
% created by Jan Hrdlicka, 16.9.2010
%-----------------------------------------------------------
% Function associationRules.m generates association rules with minimal
% confidence "minConfidence" from frequent itemsets "freqPatt"
%
% freqPatt is a Nx2 cell array with the frequent itemsets and its frequencies
% where N is number of frequent itemsets. In the first column are itemsets
% vectors, in the second are corresponding frequencies
%
% minConfidence is scalar number indicationg minimal confidence for the
% generated rules
%
% output is Mx4 cell array. M is number of the found rules. In the 1st column are antecedents of the rules,
% in the 2nd column are consequents. In the 3rd column is support and in
% the 4th confidence. 
antecedents = {};
consequents = {};
rules = {};
for i = 1:size(freqPatt)
    if length(freqPatt{i,1})>1
        antecedents{end+1,1} = freqPatt{i,1};
        consequents{end+1,1} = [];
    end
end

while ~isempty(antecedents)
    [newAntecedents,newConsequents] = newGenerationRules(antecedents,consequents);
    for i = 1:size(newAntecedents,1)
        ant = newAntecedents{i,1};
        con = newConsequents{i,1};
        join = [ant con];
        support = findSupport(join,tranDb);
        conf = support/findSupport(ant,tranDb);
        if conf>=minConfidence
            rules{end+1,1} = newAntecedents{i,1};
            rules{end,2} = newConsequents{i,1};
            rules{end,3} = support;
            rules{end,4} = conf;
        end
    end
    antecedents = newAntecedents;
    consequents = newConsequents;
end

function [newAnt, newCon] = newGenerationRules(antecedents,consequents)
%[newAnt, newCon] = newGenerationRules(antecedents,consequents)
%-----------------------------------------------------------
% created by Jan Hrdlicka, 16.9.2010
%-----------------------------------------------------------
%Creates new generation of antecedents and consequents by moving
%antecedents to consequents
newAnt = {};
newCon = {};
for i = 1:size(antecedents,1)
    ant = antecedents{i,1};
    con = consequents{i,1};
    if length(ant)==1
    else   
        for j = 1:length(ant)
            antCand = ant;
            antCand(j) = [];
            newAnt{end+1,1} = antCand;
            conCand = con;
            conCand(end+1) = ant(j);
            conCand = sort(conCand);
            newCon{end+1,1} = conCand;
        end
    end
end

function supp = findSupport(word,datafile)
%supp = findSupport(word,datafile)
%-----------------------------------------------------------
% created by Jan Hrdlicka, 16.9.2010
%-----------------------------------------------------------
% Finds relative support of the transaction(itemset) "word" in transaction DB "datafile"
% in matrix form (without counts)
% word(itemset) is vector of items
supp = datafile(:,word)==1;
supp = sum(all(supp==1,2))/size(datafile,1);